// VinciaTools.h is a part of the VINCIA plug-in to the PYTHIA 8 generator.
// Copyright (C) 2010 W.T. Giele, D.A. Kosower, P.Z. Skands.
// VINCIA is licenced under the GNU GPL version 2.
// Please respect the MCnet Guidelines for Event Generator Authors and Users, 
//               http://www.montecarlonet.org/GUIDELINES

// This file contains header information for VINCIA's Toolbox.
  
// Classes:
// VinClu : VINCIA's clustering maps
// Rambo  : Flat phase-space generator

#ifndef VINCIA_VinciaTools_H
#define VINCIA_VinciaTools_H

#include "Basics.h"             // Vec4
#include "VinciaCommon.h"       // Vec4toD4, m2(...)

using namespace Pythia8;

namespace Vincia {

  //**************************************************************************

  // VINCIA Clustering Map

  class VinClu {

  public:
    
    // Constructor.
    VinClu() {isInitPtr=false;}

    // Destructor.
    virtual ~VinClu() {}

    // Initialize pointers
    void initPtr(Rndm* rndmPtrIn) {rndmPtr = rndmPtrIn; isInitPtr=true;}
    
    // Main Vincia clustering map
    void vc3to2(int iMap,vector<Vec4>,int,int,int,vector<Vec4>&);
    
    //******

    // Individual maps, accessed by the main algorithm

    // Imap = 1 & 2 so far accessed by VC3TO2 fortran routine

    // iMap = 3: Kosower's 3->2 clustering 
    void ks3to2(Vec4 pi, Vec4 pj, Vec4 pk, Vec4& pOutI, Vec4& pOutK);

 private :
  
    // Is initialized?
    bool isInitPtr;

    // Pointer to the random number generator.
    Rndm*  rndmPtr;
    

  };

  //**************************************************************************

  // Rambo flat phase-space generator

  class Rambo {
   
  public:
    
    // Constructor.
    Rambo() {isInitPtr=false;}
    
    // Destructor.
    virtual ~Rambo() {}
    
    // Initialize pointers
    void initPtr(Rndm* rndmPtrIn) {rndmPtr = rndmPtrIn; isInitPtr=true;}
    
    // Rambo phase space generator
    // p[0][i] is the energy of the i'th parton
    double genPoint(double S,int N,double p[4][8]);
    
  private:
    
    // Is initialized?
    bool isInitPtr;

    // Pointer to the random number generator.
    Rndm*  rndmPtr;
    
  };

}

#endif
